﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Syncfusion.Windows.Tools.Controls;
using System.IO;
using System.Windows.Data;
using System.Collections;
using System.Collections.ObjectModel;
using System.Windows.Media.Imaging;

namespace OfficeUIDemo
{
    public partial class RibbonSample : UserControl
    {
        public RibbonSample()
        {
            InitializeComponent();
            DataContext = this;

            AddEvents();
        }

        internal void AddEvents()
        {
            this.rich.FileOpening += new RichTextBoxAdv.FileOpeningEventHandler(rich_FileOpening);
            this.rich.FileSaving += new RichTextBoxAdv.FileSavingEventHandler(rich_FileSaving);
            rich.MouseRightButtonUp += new MouseButtonEventHandler(rich_MouseRightButtonUp);
            rich.MouseLeftButtonUp += new MouseButtonEventHandler(rich_MouseLeftButtonUp);
            rich.MouseLeftButtonDown += new MouseButtonEventHandler(OnDown);
            rich.MouseRightButtonDown += new MouseButtonEventHandler(OnDown);
            rich.MouseMove += new MouseEventHandler(rich_MouseMove);

            if (FontFamilyCombo != null)
                FontFamilyCombo.DropDownClosed += (sender, args) =>
                {
                    if (rich != null)
                    {
                        rich.ChangeFontFamilyCommand.Execute(FontFamilyCombo.SelectedItem);
                    }
                };

            if (FontSizeCombo != null)
                FontSizeCombo.DropDownClosed += (sender, args) =>
                {
                    if (rich != null)
                    {
                        double fontsize;
                        if (double.TryParse(FontSizeCombo.SelectedValue.ToString(), out fontsize))
                        {
                            rich.ChangeFontSizeCommand.Execute(fontsize);
                        }
                    }
                };

            if (FontColorSplitButton != null)
                FontColorSplitButton.MouseLeftButtonDown += (sender, e) =>
                {
                    if (FontColorpicker != null)
                    {
                        rich.ChangeTextColorCommand.Execute(FontColorpicker.Color);
                    }
                };

            if (HighlightColorSplitButton != null)
                HighlightColorSplitButton.MouseLeftButtonDown += (sender, e) =>
                {
                    if (Highlightcolorpicker != null)
                    {
                        rich.ChangeHighlightColorCommand.Execute(Highlightcolorpicker.Color);
                    }
                };

            if (Highlightcolorpicker != null)
                Highlightcolorpicker.ColorClicked += (sender, args) =>
                {
                    if (Highlightcolorpicker != null && rich != null)
                    {
                        rich.ChangeHighlightColorCommand.Execute(Highlightcolorpicker.Color);
                        RibbonSplitButton splitbutton = (RibbonSplitButton)Highlightcolorpicker.Parent;
                        splitbutton.IsDropDownOpen = false;
                    }
                };

            if (FontColorpicker != null)
            {
                FontColorpicker.ColorChanged += (sender, args) =>
                {
                    rich.ChangeTextColorCommand.Execute(FontColorpicker.Color);
                    RibbonSplitButton splitbutton = (RibbonSplitButton)FontColorpicker.Parent;
                    splitbutton.IsDropDownOpen = false;
                };
            }

            if (btnIncFont != null)
                btnIncFont.Click += (sender, args) =>
                {
                    if (FontSizeCombo.SelectedIndex < FontSizeCombo.Items.Count - 1)
                    {
                        FontSizeCombo.SelectedIndex += 1;
                        rich.Selection.ChangeFontSize(int.Parse((FontSizeCombo.SelectedItem).ToString()));
                    }
                    else
                    {
                        rich.Selection.ChangeFontSize(rich.CurrentInlineStyle.FontSize + 10);
                    }
                };

            if (btnDecFont != null)
                btnDecFont.Click += (sender, args) =>
                {
                    if (FontSizeCombo.SelectedIndex > 0)
                    {
                        FontSizeCombo.SelectedIndex -= 1;
                        rich.Selection.ChangeFontSize(int.Parse((FontSizeCombo.SelectedItem).ToString()));
                    }
                    else
                    {
                        if (rich.CurrentInlineStyle.FontSize > 1)
                            rich.Selection.ChangeFontSize(rich.CurrentInlineStyle.FontSize - 1);
                    }
                };
        }

        void OnDown(object sender, MouseButtonEventArgs e)
        {
            if (minitoolbar != null)
            {
                minitoolbar.IsOpen = false;
            }
        }
        Point MousePos;

        void rich_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            RichTextBoxAdv owner = sender as RichTextBoxAdv;

            if (owner != null && owner.Selection.Text != string.Empty)
            {
                minitoolbar.Target = owner;
                minitoolbar.IsOpen = true;

                minitoolbar.Background = new SolidColorBrush(Colors.White);
                minitoolbar.BorderBrush = new SolidColorBrush(Colors.Gray);
                minitoolbar.BorderThickness = new Thickness(1);

                double x = MousePos.X;
                double y = MousePos.Y;

                FrameworkElement _rootVisual = Application.Current.RootVisual as FrameworkElement;

                // Try not to let it stick out too far to the right/bottom
                x = Math.Min(x, _rootVisual.ActualWidth - owner.ContextMenu.ActualWidth);
                y = Math.Min(y, _rootVisual.ActualHeight - owner.ContextMenu.ActualHeight);

                minitoolbar.HorizontalOffset = x;
                minitoolbar.VerticalOffset = y - 10;
            }
            else
                minitoolbar.IsOpen = false;

        }

        void rich_MouseMove(object sender, MouseEventArgs e)
        {
            MousePos = e.GetPosition((UIElement)sender);
        }
        
        void rich_MouseRightButtonUp(object sender, MouseButtonEventArgs e)
        {
            RichTextBoxAdv richtext = sender as RichTextBoxAdv;
            Point p = e.GetPosition((UIElement)sender);

            if (richtext != null)
            {
                minitoolbar.Target = richtext;
                minitoolbar.IsOpen = true;

                minitoolbar.Background = new SolidColorBrush(Colors.White);
                minitoolbar.BorderBrush = new SolidColorBrush(Colors.Gray);
                minitoolbar.BorderThickness = new Thickness(1);

                double x = p.X;
                double y = p.Y;

                p = PositionInBrowser.GetPosition(richtext.ContextMenu);

                minitoolbar.HorizontalOffset = x;
                minitoolbar.VerticalOffset = y - 10;
            }
            else
                minitoolbar.IsOpen = false;
        }

        void rich_FileSaving(object obj, FileSavingEventArgs args)
        {
            if (args.FormatType == ".docx" || args.FormatType == ".doc")
                DocxExporting.ConvertToDocument(this.rich.Document, args.DoucmentStream, args.FormatType);
        }

        void rich_FileOpening(object obj, FileOpeningEventArgs args)
        {
            if (args.FormatType == ".docx" || args.FormatType == ".doc")
                rich.Document = DocxImporting.ConvertToDocumentAdv(args.DocumentStream, args.FormatType);
        }

        private void fontsizecombo_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            RibbonComboBox combo = sender as RibbonComboBox;
            if (combo != null && rich != null)
            {
                rich.ChangeFontSizeCommand.Execute((combo.SelectedItem as RibbonComboBoxItem).Content);
            }
        }

        private void fontfamilycombo_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            RibbonComboBox combo = sender as RibbonComboBox;
            if (combo != null && rich != null)
            {
                rich.ChangeFontFamilyCommand.Execute((combo.SelectedItem as RibbonComboBoxItem).Content);
            }
        }

        public void Dispose()
        {
            this.rich.FileOpening -= new RichTextBoxAdv.FileOpeningEventHandler(rich_FileOpening);
            this.rich.FileSaving -= new RichTextBoxAdv.FileSavingEventHandler(rich_FileSaving);
            rich.MouseRightButtonUp -= new MouseButtonEventHandler(rich_MouseRightButtonUp);
            rich.MouseLeftButtonUp -= new MouseButtonEventHandler(rich_MouseLeftButtonUp);
            rich.MouseLeftButtonDown -= new MouseButtonEventHandler(OnDown);
            rich.MouseRightButtonDown -= new MouseButtonEventHandler(OnDown);
            rich.MouseMove -= new MouseEventHandler(rich_MouseMove);
        }
    }

    public static class PositionInBrowser
    {
        internal static Point GetPosition(this UIElement uiElement)
        {
            try
            {
                var gt = uiElement.TransformToVisual(Application.Current.RootVisual);
                var position = gt.Transform(new Point(0, 0));
                var zoomFactor = Application.Current.Host.Content.ZoomFactor;   //Adjust to Browser Zooming.
                return new Point(position.X * zoomFactor, position.Y * zoomFactor);
            }
            catch { }
            return new Point();
        }
    }

    public class ZoomFactorInPercentage : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            double factor = Math.Round(System.Convert.ToDouble(value), 2);
            if (factor == 0)
            {
                factor = 0.1;
            }
            return (int)(factor * 100 * 4);
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class FactorDivision : IValueConverter
    {

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value != null)
            {
                double no = (double)value;
                return no / 10;
            }
            return null;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
